# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Untag images command."""


from containerregistry.client import docker_name
from containerregistry.client.v2_2 import docker_session
from googlecloudsdk.api_lib.container.images import util
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.container import flags
from googlecloudsdk.core import log
from googlecloudsdk.core.console import console_io


class Untag(base.DeleteCommand):
  """Remove existing image tags.

  The container images untag command of gcloud deletes a specified
  tag on a specified image. Repositories must be hosted by the
  Google Container Registry.
  """

  detailed_help = {
      'DESCRIPTION':
          """\
          The container images untag command removes the specified tag
          from the image.
      """,
      'EXAMPLES':
          """\
          Removes the tag from the input IMAGE_NAME:

            $ {command} <IMAGE_NAME>

      """,
  }

  @staticmethod
  def Args(parser):
    """Register flags for this command.

    Args:
      parser: An argparse.ArgumentParser-like object. It is mocked out in order
        to capture some information, but behaves like an ArgumentParser.
    """
    flags.AddTagOrDigestPositional(parser, verb='untag', tags_only=True)

  def Run(self, args):
    """This is what is called when the user runs this command.

    Args:
      args: an argparse namespace. All the arguments that were provided to this
        command invocation.

    Raises:
      util.InvalidImageNameError: If the user specified an invalid
      (or non-existent) image name.
    Returns:
      A list of the deleted docker_name.Tag objects
    """
    # IMAGE_NAME: The fully-qualified image name to delete (with a tag).
    # Removes the tag from the image. Ex. gcr.io/google-appengine/java:TAG.

    http_obj = util.Http()
    # collect input/validate
    tags = self._ParseArgs(args.image_names)

    digests = dict()
    with util.WrapExpectedDockerlessErrors():
      for tag in tags:
        try:
          # Resolve tags to digests. Throws InvalidImageNameError on 404.
          digests[tag] = util.GetDigestFromName(str(tag))
        except util.InvalidImageNameError as exc:
          # We already validated the image string in _ParseArgs, this is a 404
          raise util.InvalidImageNameError(
              'Image could not be found: [{}]'.format(str(tag))
          ) from exc

      if not tags:
        log.warning('No tags found matching image names [%s].',
                    ', '.join(args.image_names))
        return
      for tag, digest in digests.items():
        log.status.Print('Tag: [{}]'.format(str(tag)))
        log.status.Print('- referencing digest: [{}]'.format(str(digest)))
        log.status.Print('')

      console_io.PromptContinue(
          'This operation will remove the above tags. '
          'Tag removals only delete the tags; '
          'The underlying image layers (referenced by the above digests) will '
          'continue to exist.',
          cancel_on_no=True)

      # delete and collect output
      result = []
      for tag in tags:
        self._DeleteDockerTag(tag, digests, http_obj)
        result.append({'name': str(tag)})
      return result

  def _ParseArgs(self, image_names):
    tags = set()
    for image_name in image_names:
      docker_obj = util.GetDockerImageFromTagOrDigest(image_name)
      if (isinstance(docker_obj, docker_name.Tag) and
          util.IsFullySpecified(image_name)):
        # Only accept explicitly named tags for removal.
        tags.add(docker_obj)
      else:
        raise util.InvalidImageNameError(
            'IMAGE_NAME must be of the form [*.gcr.io/repository:<tag>]: '
            '[{}]'.format(image_name))
    return tags

  def _DeleteDockerTag(self, tag, digests, http_obj):
    # calling Delete on an image referenced by tag only deletes the tag
    docker_session.Delete(
        creds=util.CredentialProvider(), name=tag, transport=http_obj)
    log.DeletedResource('[{tag}] (referencing [{digest}])'.format(
        tag=tag, digest=digests[tag]))



# -*- coding: utf-8 -*- #
# Copyright 2018 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""cloud-shell ssh command."""


import datetime
import shlex
import threading
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.cloud_shell import util
from googlecloudsdk.command_lib.cloud_shell.tunnel import CloudShellTunnel
from googlecloudsdk.command_lib.util.ssh import ssh
from googlecloudsdk.core import log
from googlecloudsdk.core import properties


@base.ReleaseTracks(base.ReleaseTrack.GA, base.ReleaseTrack.BETA)
@base.UniverseCompatible
class Ssh(base.Command):
  """Allows you to establish an interactive SSH session with Cloud Shell."""

  detailed_help = {
      'DESCRIPTION':
          """\
        *{command}* lets you remotely log in to Cloud Shell. If your Cloud Shell
        is not currently running, this will cause it to be started before
        establishing the SSH session.
        """,
      'EXAMPLES':
          """\
        To SSH into your Cloud Shell, run:

            $ {command}

        To run a remote command in your Cloud Shell, run:

            $ {command} --command=ls
        """,
  }

  @staticmethod
  def Args(parser):
    util.ParseCommonArgs(parser)
    parser.add_argument(
        '--command',
        help="""\
        A command to run in Cloud Shell.

        Runs the command in Cloud Shell and then exits.
        """)
    parser.add_argument(
        '--dry-run',
        help="""\
        If provided, prints the command that would be run to standard out
        instead of executing it.
        """,
        action='store_true')
    parser.add_argument(
        '--ssh-flag',
        help='Additional flags to be passed to *ssh(1)*.',
        action='append')
    parser.add_argument(
        '--authorize-session',
        help="""\
        If provided, sends OAuth credentials to the current Cloud Shell session
        on behalf of the user. When this completes, the session will be
        authorized to run various Google Cloud command-line tools without
        requiring the user to manually authenticate.
        """,
        action='store_true')

  def Run(self, args):
    if not args.authorize_session:
      log.Print('Automatic authentication with GCP CLI tools in Cloud Shell is '
                'disabled. To enable, please rerun command with '
                '`--authorize-session` flag.')
    command_list = args.command.split(' ') if args.command else ['bash -l']
    project = properties.VALUES.core.project.Get()
    connection_info = util.PrepareEnvironment(args)
    if args.authorize_session:
      util.AuthorizeEnvironment()
    if not connection_info.web_host:
      raise FileNotFoundError(
          'Failed to get Cloud Shell web host. Environment might not be fully'
          ' initialized.'
      )
    jwt = util.GenerateAccessToken()
    tunnel = CloudShellTunnel(host=connection_info.web_host, jwt=jwt)
    tunnel.Start()
    command = ssh.SSHCommand(
        remote=ssh.Remote(host='localhost', user=connection_info.user),
        port=str(tunnel.local_port),
        identity_file=connection_info.key,
        remote_command=([f'DEVSHELL_PROJECT_ID={project}'] if project else [])
        + command_list,
        extra_flags=args.ssh_flag,
        tty=not args.command,
        options={'StrictHostKeyChecking': 'no'},
        remainder=getattr(args, 'ssh_args', None),
    )
    # TODO(b/513021781): Cleanup AddPublicKey 410 fallback and direct SSH once
    # passwordless SSH is fully enforced.
    try:
      if args.dry_run:
        elems = command.Build(connection_info.ssh_env)
        log.Print(' '.join([shlex.quote(elem) for elem in elems]))
      elif args.authorize_session:
        self.done = threading.Event()
        thread = threading.Thread(target=self.Reauthorize, args=(), daemon=True)
        thread.start()
        try:
          command.Run(connection_info.ssh_env)
        except ssh.CommandError as exc:
          if connection_info.add_public_key_error:
            raise connection_info.add_public_key_error from exc
          raise
        self.done.set()
      else:
        try:
          command.Run(connection_info.ssh_env)
        except ssh.CommandError as exc:
          if connection_info.add_public_key_error:
            raise connection_info.add_public_key_error from exc
          raise
    finally:
      tunnel.Stop()

  def Reauthorize(self):
    while not self.done.is_set():
      self.done.wait(
          (
              util.MIN_CREDS_EXPIRY - datetime.timedelta(minutes=2)
          ).total_seconds()
      )
      if not self.done.is_set():
        util.AuthorizeEnvironment()


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class SshAlpha(Ssh):
  """Allows you to establish an interactive SSH session with Cloud Shell."""

  @staticmethod
  def Args(parser):
    Ssh.Args(parser)
    util.AddSshArgFlag(parser)



# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Fleet API type helpers.

Because the Fleet API is split into multiple API tracks, this file provides
helpers to make it easier to work with the different tracks. TypeAlias is not
used because it is only supported in Python 3.10+. These type aliases are
intended to be used in type hints when the specific track is not known.
"""

from typing import Generator

from googlecloudsdk.generated_clients.apis.gkehub.v1 import gkehub_v1_client as ga_client
from googlecloudsdk.generated_clients.apis.gkehub.v1 import gkehub_v1_messages as ga_messages
from googlecloudsdk.generated_clients.apis.gkehub.v1alpha import gkehub_v1alpha_client as alpha_client
from googlecloudsdk.generated_clients.apis.gkehub.v1alpha import gkehub_v1alpha_messages as alpha_messages
from googlecloudsdk.generated_clients.apis.gkehub.v1beta import gkehub_v1beta_client as beta_client
from googlecloudsdk.generated_clients.apis.gkehub.v1beta import gkehub_v1beta_messages as beta_messages
from googlecloudsdk.generated_clients.apis.gkehub.v2 import gkehub_v2_client as v2_ga_client
from googlecloudsdk.generated_clients.apis.gkehub.v2alpha import gkehub_v2alpha_client as v2_alpha_client
from googlecloudsdk.generated_clients.apis.gkehub.v2beta import gkehub_v2beta_client as v2_beta_client


BinaryAuthorizationConfig = (
    alpha_messages.BinaryAuthorizationConfig
    | beta_messages.BinaryAuthorizationConfig
    | ga_messages.BinaryAuthorizationConfig
)

BinaryAuthorizationConfigEvaluationModeValueValuesEnum = (
    alpha_messages.BinaryAuthorizationConfig.EvaluationModeValueValuesEnum
    | beta_messages.BinaryAuthorizationConfig.EvaluationModeValueValuesEnum
    | ga_messages.BinaryAuthorizationConfig.EvaluationModeValueValuesEnum
)

CompliancePostureConfig = (
    alpha_messages.CompliancePostureConfig
    | beta_messages.CompliancePostureConfig
    | ga_messages.CompliancePostureConfig
)

DefaultClusterConfig = (
    alpha_messages.DefaultClusterConfig
    | beta_messages.DefaultClusterConfig
    | ga_messages.DefaultClusterConfig
)

Fleet = alpha_messages.Fleet | beta_messages.Fleet | ga_messages.Fleet

GkehubProjectsLocationsFleetsCreateRequest = (
    alpha_messages.GkehubProjectsLocationsFleetsCreateRequest
    | beta_messages.GkehubProjectsLocationsFleetsCreateRequest
    | ga_messages.GkehubProjectsLocationsFleetsCreateRequest
)

GkehubProjectsLocationsFleetsDeleteRequest = (
    alpha_messages.GkehubProjectsLocationsFleetsDeleteRequest
    | beta_messages.GkehubProjectsLocationsFleetsDeleteRequest
    | ga_messages.GkehubProjectsLocationsFleetsDeleteRequest
)

GkehubProjectsLocationsFleetsPatchRequest = (
    alpha_messages.GkehubProjectsLocationsFleetsPatchRequest
    | beta_messages.GkehubProjectsLocationsFleetsPatchRequest
    | ga_messages.GkehubProjectsLocationsFleetsPatchRequest
)

GkehubProjectsLocationsOperationsListRequest = (
    alpha_messages.GkehubProjectsLocationsOperationsListRequest
    | beta_messages.GkehubProjectsLocationsOperationsListRequest
    | ga_messages.GkehubProjectsLocationsOperationsListRequest
)

GkehubProjectsLocationsOperationsGetRequest = (
    alpha_messages.GkehubProjectsLocationsOperationsGetRequest
    | beta_messages.GkehubProjectsLocationsOperationsGetRequest
    | ga_messages.GkehubProjectsLocationsOperationsGetRequest
)

GkehubProjectsLocationsRolloutsCreateRequest = (
    alpha_messages.GkehubProjectsLocationsRolloutsCreateRequest
    # Rollouts are not yet available in beta or GA.
    # | beta_messages.GkehubProjectsLocationsRolloutsCreateRequest
    # | ga_messages.GkehubProjectsLocationsRolloutsCreateRequest
)

GkehubProjectsLocationsRolloutsDeleteRequest = (
    alpha_messages.GkehubProjectsLocationsRolloutsDeleteRequest
    | beta_messages.GkehubProjectsLocationsRolloutsDeleteRequest
    | ga_messages.GkehubProjectsLocationsRolloutsDeleteRequest
)

GkehubProjectsLocationsRolloutsForceCompleteStageRequest = (
    alpha_messages.GkehubProjectsLocationsRolloutsForceCompleteStageRequest
    | beta_messages.GkehubProjectsLocationsRolloutsForceCompleteStageRequest
    | ga_messages.GkehubProjectsLocationsRolloutsForceCompleteStageRequest
)

GkehubProjectsLocationsRolloutsCancelRequest = (
    alpha_messages.GkehubProjectsLocationsRolloutsCancelRequest
    | beta_messages.GkehubProjectsLocationsRolloutsCancelRequest
    | ga_messages.GkehubProjectsLocationsRolloutsCancelRequest
)

GkehubProjectsLocationsRolloutsGetRequest = (
    alpha_messages.GkehubProjectsLocationsRolloutsGetRequest
    | beta_messages.GkehubProjectsLocationsRolloutsGetRequest
    | ga_messages.GkehubProjectsLocationsRolloutsGetRequest
)

GkehubProjectsLocationsRolloutsListRequest = (
    alpha_messages.GkehubProjectsLocationsRolloutsListRequest
    | beta_messages.GkehubProjectsLocationsRolloutsListRequest
    | ga_messages.GkehubProjectsLocationsRolloutsListRequest
)

GkehubProjectsLocationsRolloutsPauseRequest = (
    alpha_messages.GkehubProjectsLocationsRolloutsPauseRequest
    | beta_messages.GkehubProjectsLocationsRolloutsPauseRequest
    | ga_messages.GkehubProjectsLocationsRolloutsPauseRequest
)

GkehubProjectsLocationsRolloutsResumeRequest = (
    alpha_messages.GkehubProjectsLocationsRolloutsResumeRequest
    | beta_messages.GkehubProjectsLocationsRolloutsResumeRequest
    | ga_messages.GkehubProjectsLocationsRolloutsResumeRequest
)

GkehubProjectsLocationsRolloutSequencesCreateRequest = (
    alpha_messages.GkehubProjectsLocationsRolloutSequencesCreateRequest
    | beta_messages.GkehubProjectsLocationsRolloutSequencesCreateRequest
    | ga_messages.GkehubProjectsLocationsRolloutSequencesCreateRequest
)

GkehubProjectsLocationsRolloutSequencesGetRequest = (
    alpha_messages.GkehubProjectsLocationsRolloutSequencesGetRequest
    | beta_messages.GkehubProjectsLocationsRolloutSequencesGetRequest
    | ga_messages.GkehubProjectsLocationsRolloutSequencesGetRequest
)

GkehubProjectsLocationsRolloutSequencesListRequest = (
    alpha_messages.GkehubProjectsLocationsRolloutSequencesListRequest
    | beta_messages.GkehubProjectsLocationsRolloutSequencesListRequest
    | ga_messages.GkehubProjectsLocationsRolloutSequencesListRequest
)

GkehubProjectsLocationsRolloutSequencesDeleteRequest = (
    alpha_messages.GkehubProjectsLocationsRolloutSequencesDeleteRequest
    | beta_messages.GkehubProjectsLocationsRolloutSequencesDeleteRequest
    | ga_messages.GkehubProjectsLocationsRolloutSequencesDeleteRequest
)

GkehubProjectsLocationsRolloutSequencesUpgradeRequest = (
    alpha_messages.GkehubProjectsLocationsRolloutSequencesUpgradeRequest
    | beta_messages.GkehubProjectsLocationsRolloutSequencesUpgradeRequest
    | ga_messages.GkehubProjectsLocationsRolloutSequencesUpgradeRequest
)

GkehubProjectsLocationsRolloutSequencesPatchRequest = (
    alpha_messages.GkehubProjectsLocationsRolloutSequencesPatchRequest
    | beta_messages.GkehubProjectsLocationsRolloutSequencesPatchRequest
    | ga_messages.GkehubProjectsLocationsRolloutSequencesPatchRequest
)

Operation = (
    alpha_messages.Operation
    | beta_messages.Operation
    | ga_messages.Operation
)

PolicyBinding = (
    alpha_messages.PolicyBinding
    | beta_messages.PolicyBinding
    | ga_messages.PolicyBinding
)

Rollout = (
    alpha_messages.Rollout
    | beta_messages.Rollout
    | ga_messages.Rollout
)

RolloutSequence = (
    alpha_messages.RolloutSequence
    | beta_messages.RolloutSequence
    | ga_messages.RolloutSequence
)

RolloutGenerator = Generator[Rollout, None, None]

RolloutSequenceGenerator = Generator[RolloutSequence, None, None]

SecurityPostureConfig = (
    alpha_messages.SecurityPostureConfig
    | beta_messages.SecurityPostureConfig
    | ga_messages.SecurityPostureConfig
)

SecurityPostureConfigModeValueValuesEnum = (
    alpha_messages.SecurityPostureConfig.ModeValueValuesEnum
    | beta_messages.SecurityPostureConfig.ModeValueValuesEnum
    | ga_messages.SecurityPostureConfig.ModeValueValuesEnum
)

SecurityPostureConfigVulnerabilityModeValueValuesEnum = (
    alpha_messages.SecurityPostureConfig.VulnerabilityModeValueValuesEnum
    | beta_messages.SecurityPostureConfig.VulnerabilityModeValueValuesEnum
    | ga_messages.SecurityPostureConfig.VulnerabilityModeValueValuesEnum
)

TrackClient = (
    alpha_client.GkehubV1alpha | beta_client.GkehubV1beta | ga_client.GkehubV1
)

V2TrackClient = (
    v2_alpha_client.GkehubV2alpha
    | v2_beta_client.GkehubV2beta
    | v2_ga_client.GkehubV2
)



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Flags for Firewall Plus Endpoint commands."""


from googlecloudsdk.api_lib.network_security.firewall_endpoints import activation_api
from googlecloudsdk.calliope import actions
from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope.concepts import concepts
from googlecloudsdk.calliope.concepts import deps as deps_lib
from googlecloudsdk.calliope.concepts import multitype
from googlecloudsdk.command_lib.util.concepts import concept_parsers
from googlecloudsdk.command_lib.util.concepts import presentation_specs
from googlecloudsdk.core import properties


ENDPOINT_RESOURCE_NAME = "FIREWALL_ENDPOINT"
ORG_ENDPOINT_RESOURCE_COLLECTION = (
    "networksecurity.organizations.locations.firewallEndpoints"
)
PROJECT_ENDPOINT_RESOURCE_COLLECTION = (
    "networksecurity.projects.locations.firewallEndpoints"
)
BILLING_HELP_TEST = (
    "The Google Cloud project ID to use for API enablement check, quota, and"
    " endpoint uptime billing."
)


def OrgEndpointResourceSpec(api_version):
  """Returns the resource spec for an organization level firewall endpoint."""
  return concepts.ResourceSpec(
      ORG_ENDPOINT_RESOURCE_COLLECTION,
      "firewall endpoint",
      api_version=api_version,
      organizationsId=concepts.ResourceParameterAttributeConfig(
          "organization",
          "Organization ID of the {resource}.",
          parameter_name="organizationsId",
      ),
      locationsId=concepts.ResourceParameterAttributeConfig(
          "zone",
          "Zone of the {resource}.",
          parameter_name="locationsId",
          fallthroughs=[
              deps_lib.ArgFallthrough("--location"),
          ],
      ),
      firewallEndpointsId=concepts.ResourceParameterAttributeConfig(
          "endpoint-name",
          "Name of the {resource}",
          parameter_name="firewallEndpointsId",
      ),
  )


def ProjectEndpointResourceSpec(api_version):
  """Returns the resource spec for a project level firewall endpoint."""
  return concepts.ResourceSpec(
      PROJECT_ENDPOINT_RESOURCE_COLLECTION,
      "firewall endpoint",
      api_version=api_version,
      projectsId=concepts.ResourceParameterAttributeConfig(
          name="project",
          help_text=(
              "Project ID of the Google Cloud project for the {resource}."
          ),
          fallthroughs=[
              # Do not fallthrough to the --project flag, as this will prompt
              # the user to choose between project and org scoped endpoints when
              # supplying both --organization and --project.
              deps_lib.PropertyFallthrough(properties.VALUES.core.project),
          ],
      ),
      locationsId=concepts.ResourceParameterAttributeConfig(
          "zone",
          "Zone of the {resource}.",
          parameter_name="locationsId",
          fallthroughs=[
              deps_lib.ArgFallthrough("--location"),
          ],
      ),
      firewallEndpointsId=concepts.ResourceParameterAttributeConfig(
          "endpoint-name",
          "Name of the {resource}",
          parameter_name="firewallEndpointsId",
      ),
  )


def AddEndpointResource(release_track, parser, project_scope_supported=False):
  """Adds Firewall Plus endpoint resource."""
  api_version = activation_api.GetApiVersion(release_track)
  concept_specs = [
      OrgEndpointResourceSpec(api_version),
  ]
  if project_scope_supported:
    concept_specs.append(ProjectEndpointResourceSpec(api_version))
  resource_spec = multitype.MultitypeResourceSpec(
      "firewall endpoint",
      *concept_specs,
      allow_inactive=True,
  )
  presentation_spec = presentation_specs.MultitypeResourcePresentationSpec(
      name=ENDPOINT_RESOURCE_NAME,
      concept_spec=resource_spec,
      required=True,
      group_help="Firewall Plus.",
  )
  return concept_parsers.ConceptParser([presentation_spec]).AddToParser(parser)


def AddMaxWait(
    parser,
    default_max_wait,
    help_text="Time to synchronously wait for the operation to complete, after which the operation continues asynchronously. Ignored if --no-async isn't specified. See $ gcloud topic datetimes for information on time formats.",
):
  """Adds --max-wait flag."""
  parser.add_argument(
      "--max-wait",
      dest="max_wait",
      required=False,
      default=default_max_wait,
      help=help_text,
      type=arg_parsers.Duration(),
  )


def MakeGetUriFunc(release_track):
  return (
      lambda x: activation_api.GetEffectiveApiEndpoint(release_track) + x.name
  )


def AddOrganizationArg(parser, help_text="Organization of the endpoint"):
  parser.add_argument("--organization", required=True, help=help_text)


def AddDescriptionArg(parser, help_text="Description of the endpoint"):
  parser.add_argument("--description", required=False, help=help_text)


def AddLocationArg(
    parser, required=False, help_text="Location of the endpoint"
):
  parser.add_argument("--location", required=required, help=help_text)


def AddTargetFirewallAttachmentArg(
    parser,
    help_text="Target firewall attachment where third party endpoint forwards traffic."
):
  parser.add_argument(
      "--target-firewall-attachment", required=False, help=help_text
  )


def AddEnableJumboFramesArg(
    parser,
    required=False,
    help_text="Enable jumbo frames for the firewall endpoint. To disable jumbo frames, use --no-enable-jumbo-frames.",
):
  parser.add_argument(
      "--enable-jumbo-frames",
      required=required,
      help=help_text,
      action="store_true",
  )


def AddZoneArg(parser, required=True, help_text="Zone of the endpoint"):
  parser.add_argument("--zone", required=required, help=help_text)


def AddBillingProjectArg(
    parser,
    required=True,
    help_text=BILLING_HELP_TEST,
):
  """Add billing project argument to parser.

  Args:
    parser: ArgumentInterceptor, An argparse parser.
    required: bool, whether to make this argument required.
    help_text: str, help text to overwrite the generic --billing-project help
      text.
  """
  parser.add_argument(
      "--billing-project",
      required=required,
      help=help_text,
      action=actions.StoreProperty(properties.VALUES.billing.quota_project),
  )


# We use the explicit --update-billing-project flag as opposed to the existent
# --billing-project flag because otherwise there will be an ambiguity when a
# user wants to update other things, but not the billing project.
# For example, to update the labels, a billing project is still needed for API
# quota, making the ambiguous call:
# gcloud network-security firewall-endpoints update \
#     --billing-project=proj --update-labels=k1=v1
# This is a common use for other gcloud update flags as well.
def AddUpdateBillingProjectArg(
    parser,
    required=False,
    help_text=BILLING_HELP_TEST,
):
  """Add update billing project argument to parser.

  Args:
    parser: ArgumentInterceptor, An argparse parser.
    required: bool, whether to make this argument required.
    help_text: str, help text to display on the --update-billing-project help
      text.
  """
  parser.add_argument(
      "--update-billing-project",
      required=required,
      help=help_text,
      metavar="BILLING_PROJECT",
      action=actions.StoreProperty(properties.VALUES.billing.quota_project),
  )


def AddEnableWildfireArg(
    parser,
):
  """Adds --enable-wildfire flag.

  It corresponds to the proto field:
  google.cloud.networksecurity.v1main.FirewallEndpoint.WildfireSettings.enabled

  Args:
    parser: ArgumentInterceptor, An argparse parser.
  """
  parser.add_argument(
      "--enable-wildfire",
      action="store_true",
      required=False,
      help=(
          "Enable WildFire functionality on the endpoint. Use"
          " --enable-wildfire to enable. To disable, use --no-enable-wildfire."
      ),
  )


def AddWildfireRegionArg(
    parser,
):
  """Adds --wildfire-region flag."""
  parser.add_argument(
      "--wildfire-region",
      required=False,
      help=(
          "The region WildFire submissions from this endpoint will be sent to"
          " for analysis by WildFire. Defaults to the nearest available region."
      ),
  )


def AddContentCloudRegionArg(
    parser,
):
  """Adds --content-cloud-region flag."""
  parser.add_argument(
      "--content-cloud-region",
      required=False,
      help=(
          "The content cloud region the endpoint will use. Defaults to the"
          " nearest available region."
      ),
  )


def AddWildfireLookupTimeoutArg(
    parser,
):
  """Adds --wildfire-lookup-timeout flag.

  It corresponds to the proto field:
  google.cloud.networksecurity.v1main.FirewallEndpoint.WildfireSettings
  .wildfire_realtime_lookup_duration.

  Args:
    parser: ArgumentInterceptor, An argparse parser.
  """
  parser.add_argument(
      "--wildfire-lookup-timeout",
      type=int,
      required=False,
      help=(
          "The timeout (in milliseconds) to hold a file while the WildFire real"
          " time signature cloud performs a signature lookup."
      ),
  )


def AddWildfireLookupActionArg(
    parser,
):
  """Adds --wildfire-lookup-action flag.

  It corresponds to the proto field:
  google.cloud.networksecurity.v1main.FirewallEndpoint.WildfireSettings
  .wildfire_realtime_lookup_timeout_action.

  Args:
    parser: ArgumentInterceptor, An argparse parser.
  """
  parser.add_argument(
      "--wildfire-lookup-action",
      choices=["ALLOW", "DENY"],
      required=False,
      help=(
          "The action to take on WildFire real time signature lookup timeout."
      ),
  )


def AddWildfireAnalysisTimeoutArg(
    parser,
):
  """Adds --wildfire-analysis-timeout flag.

  It corresponds to the proto field:
  google.cloud.networksecurity.v1main.FirewallEndpoint.WildfireSettings.WildfireInlineCloudAnalysisSettings
  .max_analysis_duration.

  Args:
    parser: ArgumentInterceptor, An argparse parser.
  """
  parser.add_argument(
      "--wildfire-analysis-timeout",
      type=int,
      required=False,
      help=(
          "The timeout (in milliseconds) on a file being held while WildFire"
          " inline cloud analysis is performed."
      ),
  )


def AddWildfireAnalysisActionArg(
    parser,
):
  """Adds --wildfire-analysis-action flag.

  It corresponds to the proto field:
  google.cloud.networksecurity.v1main.FirewallEndpoint.WildfireSettings.WildfireInlineCloudAnalysisSettings.timeout_action.

  Args:
    parser: ArgumentInterceptor, An argparse parser.
  """
  parser.add_argument(
      "--wildfire-analysis-action",
      choices=["ALLOW", "DENY"],
      required=False,
      help="The action to take on WildFire inline cloud analysis timeout.",
  )


def AddEnableWildfireAnalysisLoggingArg(
    parser,
):
  """Adds enable-wildfire-analysis-logging flag.

  It corresponds to the proto field:
  google.cloud.networksecurity.v1main.FirewallEndpoint.WildfireSettings.WildfireInlineCloudAnalysisSettings
  .timeout_logging_disabled.

  Args:
    parser: ArgumentInterceptor, An argparse parser.
  """
  parser.add_argument(
      "--enable-wildfire-analysis-logging",
      action="store_true",
      required=False,
      help=(
          "Enable WildFire inline cloud analysis submission timeout logging."
          " This is enabled by default. Use"
          " `--no-enable-wildfire-analysis-logging` to disable."
      ),
  )


def AddBlockPartialHttpArg(
    parser,
):
  """Adds --block-partial-http flag.

  It corresponds to the proto field:
  google.cloud.networksecurity.v1main.FirewallEndpoint.EndpointSettings
  .http_partial_response_blocked.

  Args:
    parser: ArgumentInterceptor, An argparse parser.
  """
  parser.add_argument(
      "--block-partial-http",
      action="store_true",
      required=False,
      help=(
          "Block HTTP partial responses. Defaults to false. Use"
          " `--block-partial-http` to enable. To disable, use"
          " `--no-block-partial-http`."
      ),
  )



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Helper module to create connection profiles for a database migration."""


from googlecloudsdk.api_lib.database_migration import api_util
from googlecloudsdk.api_lib.database_migration import connection_profiles
from googlecloudsdk.core import log


class CreateHelper:
  """Helper class to create connection profiles for a database migration."""

  def create(self, release_track, parent_ref, connection_profile_ref, args,
             cp_type):
    """Create a connection profile and wait for its LRO to complete, if necessary.
    """
    location = connection_profile_ref.locationsId
    cp_client = connection_profiles.ConnectionProfilesClient(
        release_track, location=location
    )
    result_operation = cp_client.Create(
        parent_ref, connection_profile_ref.connectionProfilesId, cp_type, args)

    client = api_util.GetClientInstance(release_track, location=location)
    messages = api_util.GetMessagesModule(release_track)
    resource_parser = api_util.GetResourceParser(release_track)

    if args.IsKnownAndSpecified('no_async'):
      log.status.Print(
          'Waiting for connection profile [{}] to be created with [{}]'.format(
              connection_profile_ref.connectionProfilesId,
              result_operation.name))

      api_util.HandleLRO(client, result_operation,
                         client.projects_locations_connectionProfiles)

      log.status.Print('Created connection profile {} [{}]'.format(
          connection_profile_ref.connectionProfilesId, result_operation.name))
      return

    operation_ref = resource_parser.Create(
        'datamigration.projects.locations.operations',
        operationsId=result_operation.name,
        projectsId=connection_profile_ref.projectsId,
        locationsId=connection_profile_ref.locationsId)

    return client.projects_locations_operations.Get(
        messages.DatamigrationProjectsLocationsOperationsGetRequest(
            name=operation_ref.operationsId))



"""Generated client library for datalineage version v1."""
# NOTE: This file is autogenerated and should not be edited by hand.

from apitools.base.py import base_api
from googlecloudsdk.generated_clients.apis.datalineage.v1 import datalineage_v1_messages as messages


class DatalineageV1(base_api.BaseApiClient):
  """Generated client library for service datalineage version v1."""

  MESSAGES_MODULE = messages
  BASE_URL = 'https://datalineage.googleapis.com/'
  MTLS_BASE_URL = 'https://datalineage.mtls.googleapis.com/'

  _PACKAGE = 'datalineage'
  _SCOPES = ['https://www.googleapis.com/auth/cloud-platform', 'https://www.googleapis.com/auth/datalineage.read-write', 'https://www.googleapis.com/auth/datalineage.readonly']
  _VERSION = 'v1'
  _CLIENT_ID = 'CLIENT_ID'
  _CLIENT_SECRET = 'CLIENT_SECRET'
  _USER_AGENT = 'google-cloud-sdk'
  _CLIENT_CLASS_NAME = 'DatalineageV1'
  _URL_VERSION = 'v1'
  _API_KEY = None

  def __init__(self, url='', credentials=None,
               get_credentials=True, http=None, model=None,
               log_request=False, log_response=False,
               credentials_args=None, default_global_params=None,
               additional_http_headers=None, response_encoding=None):
    """Create a new datalineage handle."""
    url = url or self.BASE_URL
    super(DatalineageV1, self).__init__(
        url, credentials=credentials,
        get_credentials=get_credentials, http=http, model=model,
        log_request=log_request, log_response=log_response,
        credentials_args=credentials_args,
        default_global_params=default_global_params,
        additional_http_headers=additional_http_headers,
        response_encoding=response_encoding)
    self.folders_locations_config = self.FoldersLocationsConfigService(self)
    self.folders_locations = self.FoldersLocationsService(self)
    self.folders = self.FoldersService(self)
    self.organizations_locations_config = self.OrganizationsLocationsConfigService(self)
    self.organizations_locations = self.OrganizationsLocationsService(self)
    self.organizations = self.OrganizationsService(self)
    self.projects_locations_config = self.ProjectsLocationsConfigService(self)
    self.projects_locations_operations = self.ProjectsLocationsOperationsService(self)
    self.projects_locations_processes_runs_lineageEvents = self.ProjectsLocationsProcessesRunsLineageEventsService(self)
    self.projects_locations_processes_runs = self.ProjectsLocationsProcessesRunsService(self)
    self.projects_locations_processes = self.ProjectsLocationsProcessesService(self)
    self.projects_locations = self.ProjectsLocationsService(self)
    self.projects = self.ProjectsService(self)

  class FoldersLocationsConfigService(base_api.BaseApiService):
    """Service class for the folders_locations_config resource."""

    _NAME = 'folders_locations_config'

    def __init__(self, client):
      super(DatalineageV1.FoldersLocationsConfigService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Get the Config for a given resource.

      Args:
        request: (DatalineageFoldersLocationsConfigGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudDatacatalogLineageConfigmanagementV1Config) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/folders/{foldersId}/locations/{locationsId}/config',
        http_method='GET',
        method_id='datalineage.folders.locations.config.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='DatalineageFoldersLocationsConfigGetRequest',
        response_type_name='GoogleCloudDatacatalogLineageConfigmanagementV1Config',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Update the Config for a given resource.

      Args:
        request: (GoogleCloudDatacatalogLineageConfigmanagementV1Config) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudDatacatalogLineageConfigmanagementV1Config) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/folders/{foldersId}/locations/{locationsId}/config',
        http_method='PATCH',
        method_id='datalineage.folders.locations.config.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='<request>',
        request_type_name='GoogleCloudDatacatalogLineageConfigmanagementV1Config',
        response_type_name='GoogleCloudDatacatalogLineageConfigmanagementV1Config',
        supports_download=False,
    )

  class FoldersLocationsService(base_api.BaseApiService):
    """Service class for the folders_locations resource."""

    _NAME = 'folders_locations'

    def __init__(self, client):
      super(DatalineageV1.FoldersLocationsService, self).__init__(client)
      self._upload_configs = {
          }

  class FoldersService(base_api.BaseApiService):
    """Service class for the folders resource."""

    _NAME = 'folders'

    def __init__(self, client):
      super(DatalineageV1.FoldersService, self).__init__(client)
      self._upload_configs = {
          }

  class OrganizationsLocationsConfigService(base_api.BaseApiService):
    """Service class for the organizations_locations_config resource."""

    _NAME = 'organizations_locations_config'

    def __init__(self, client):
      super(DatalineageV1.OrganizationsLocationsConfigService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Get the Config for a given resource.

      Args:
        request: (DatalineageOrganizationsLocationsConfigGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudDatacatalogLineageConfigmanagementV1Config) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/organizations/{organizationsId}/locations/{locationsId}/config',
        http_method='GET',
        method_id='datalineage.organizations.locations.config.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='DatalineageOrganizationsLocationsConfigGetRequest',
        response_type_name='GoogleCloudDatacatalogLineageConfigmanagementV1Config',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Update the Config for a given resource.

      Args:
        request: (GoogleCloudDatacatalogLineageConfigmanagementV1Config) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudDatacatalogLineageConfigmanagementV1Config) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/organizations/{organizationsId}/locations/{locationsId}/config',
        http_method='PATCH',
        method_id='datalineage.organizations.locations.config.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='<request>',
        request_type_name='GoogleCloudDatacatalogLineageConfigmanagementV1Config',
        response_type_name='GoogleCloudDatacatalogLineageConfigmanagementV1Config',
        supports_download=False,
    )

  class OrganizationsLocationsService(base_api.BaseApiService):
    """Service class for the organizations_locations resource."""

    _NAME = 'organizations_locations'

    def __init__(self, client):
      super(DatalineageV1.OrganizationsLocationsService, self).__init__(client)
      self._upload_configs = {
          }

  class OrganizationsService(base_api.BaseApiService):
    """Service class for the organizations resource."""

    _NAME = 'organizations'

    def __init__(self, client):
      super(DatalineageV1.OrganizationsService, self).__init__(client)
      self._upload_configs = {
          }

  class ProjectsLocationsConfigService(base_api.BaseApiService):
    """Service class for the projects_locations_config resource."""

    _NAME = 'projects_locations_config'

    def __init__(self, client):
      super(DatalineageV1.ProjectsLocationsConfigService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Get the Config for a given resource.

      Args:
        request: (DatalineageProjectsLocationsConfigGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudDatacatalogLineageConfigmanagementV1Config) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/config',
        http_method='GET',
        method_id='datalineage.projects.locations.config.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='DatalineageProjectsLocationsConfigGetRequest',
        response_type_name='GoogleCloudDatacatalogLineageConfigmanagementV1Config',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Update the Config for a given resource.

      Args:
        request: (GoogleCloudDatacatalogLineageConfigmanagementV1Config) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudDatacatalogLineageConfigmanagementV1Config) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/config',
        http_method='PATCH',
        method_id='datalineage.projects.locations.config.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='<request>',
        request_type_name='GoogleCloudDatacatalogLineageConfigmanagementV1Config',
        response_type_name='GoogleCloudDatacatalogLineageConfigmanagementV1Config',
        supports_download=False,
    )

  class ProjectsLocationsOperationsService(base_api.BaseApiService):
    """Service class for the projects_locations_operations resource."""

    _NAME = 'projects_locations_operations'

    def __init__(self, client):
      super(DatalineageV1.ProjectsLocationsOperationsService, self).__init__(client)
      self._upload_configs = {
          }

    def Cancel(self, request, global_params=None):
      r"""Starts asynchronous cancellation on a long-running operation. The server makes a best effort to cancel the operation, but success is not guaranteed. If the server doesn't support this method, it returns `google.rpc.Code.UNIMPLEMENTED`. Clients can use Operations.GetOperation or other methods to check whether the cancellation succeeded or whether the operation completed despite cancellation. On successful cancellation, the operation is not deleted; instead, it becomes an operation with an Operation.error value with a google.rpc.Status.code of `1`, corresponding to `Code.CANCELLED`.

      Args:
        request: (DatalineageProjectsLocationsOperationsCancelRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleProtobufEmpty) The response message.
      """
      config = self.GetMethodConfig('Cancel')
      return self._RunMethod(
          config, request, global_params=global_params)

    Cancel.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/operations/{operationsId}:cancel',
        http_method='POST',
        method_id='datalineage.projects.locations.operations.cancel',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}:cancel',
        request_field='googleLongrunningCancelOperationRequest',
        request_type_name='DatalineageProjectsLocationsOperationsCancelRequest',
        response_type_name='GoogleProtobufEmpty',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a long-running operation. This method indicates that the client is no longer interested in the operation result. It does not cancel the operation. If the server doesn't support this method, it returns `google.rpc.Code.UNIMPLEMENTED`.

      Args:
        request: (DatalineageProjectsLocationsOperationsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleProtobufEmpty) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/operations/{operationsId}',
        http_method='DELETE',
        method_id='datalineage.projects.locations.operations.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='DatalineageProjectsLocationsOperationsDeleteRequest',
        response_type_name='GoogleProtobufEmpty',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets the latest state of a long-running operation. Clients can use this method to poll the operation result at intervals as recommended by the API service.

      Args:
        request: (DatalineageProjectsLocationsOperationsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleLongrunningOperation) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/operations/{operationsId}',
        http_method='GET',
        method_id='datalineage.projects.locations.operations.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='DatalineageProjectsLocationsOperationsGetRequest',
        response_type_name='GoogleLongrunningOperation',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists operations that match the specified filter in the request. If the server doesn't support this method, it returns `UNIMPLEMENTED`.

      Args:
        request: (DatalineageProjectsLocationsOperationsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleLongrunningListOperationsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/operations',
        http_method='GET',
        method_id='datalineage.projects.locations.operations.list',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['filter', 'pageSize', 'pageToken', 'returnPartialSuccess'],
        relative_path='v1/{+name}/operations',
        request_field='',
        request_type_name='DatalineageProjectsLocationsOperationsListRequest',
        response_type_name='GoogleLongrunningListOperationsResponse',
        supports_download=False,
    )

  class ProjectsLocationsProcessesRunsLineageEventsService(base_api.BaseApiService):
    """Service class for the projects_locations_processes_runs_lineageEvents resource."""

    _NAME = 'projects_locations_processes_runs_lineageEvents'

    def __init__(self, client):
      super(DatalineageV1.ProjectsLocationsProcessesRunsLineageEventsService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a new lineage event.

      Args:
        request: (DatalineageProjectsLocationsProcessesRunsLineageEventsCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudDatacatalogLineageV1LineageEvent) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/processes/{processesId}/runs/{runsId}/lineageEvents',
        http_method='POST',
        method_id='datalineage.projects.locations.processes.runs.lineageEvents.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['requestId'],
        relative_path='v1/{+parent}/lineageEvents',
        request_field='googleCloudDatacatalogLineageV1LineageEvent',
        request_type_name='DatalineageProjectsLocationsProcessesRunsLineageEventsCreateRequest',
        response_type_name='GoogleCloudDatacatalogLineageV1LineageEvent',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes the lineage event with the specified name.

      Args:
        request: (DatalineageProjectsLocationsProcessesRunsLineageEventsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleProtobufEmpty) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/processes/{processesId}/runs/{runsId}/lineageEvents/{lineageEventsId}',
        http_method='DELETE',
        method_id='datalineage.projects.locations.processes.runs.lineageEvents.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['allowMissing'],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='DatalineageProjectsLocationsProcessesRunsLineageEventsDeleteRequest',
        response_type_name='GoogleProtobufEmpty',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets details of a specified lineage event.

      Args:
        request: (DatalineageProjectsLocationsProcessesRunsLineageEventsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudDatacatalogLineageV1LineageEvent) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/processes/{processesId}/runs/{runsId}/lineageEvents/{lineageEventsId}',
        http_method='GET',
        method_id='datalineage.projects.locations.processes.runs.lineageEvents.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='DatalineageProjectsLocationsProcessesRunsLineageEventsGetRequest',
        response_type_name='GoogleCloudDatacatalogLineageV1LineageEvent',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists lineage events in the given project and location. The list order is not defined.

      Args:
        request: (DatalineageProjectsLocationsProcessesRunsLineageEventsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudDatacatalogLineageV1ListLineageEventsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/processes/{processesId}/runs/{runsId}/lineageEvents',
        http_method='GET',
        method_id='datalineage.projects.locations.processes.runs.lineageEvents.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['pageSize', 'pageToken'],
        relative_path='v1/{+parent}/lineageEvents',
        request_field='',
        request_type_name='DatalineageProjectsLocationsProcessesRunsLineageEventsListRequest',
        response_type_name='GoogleCloudDatacatalogLineageV1ListLineageEventsResponse',
        supports_download=False,
    )

  class ProjectsLocationsProcessesRunsService(base_api.BaseApiService):
    """Service class for the projects_locations_processes_runs resource."""

    _NAME = 'projects_locations_processes_runs'

    def __init__(self, client):
      super(DatalineageV1.ProjectsLocationsProcessesRunsService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a new run.

      Args:
        request: (DatalineageProjectsLocationsProcessesRunsCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudDatacatalogLineageV1Run) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/processes/{processesId}/runs',
        http_method='POST',
        method_id='datalineage.projects.locations.processes.runs.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['requestId'],
        relative_path='v1/{+parent}/runs',
        request_field='googleCloudDatacatalogLineageV1Run',
        request_type_name='DatalineageProjectsLocationsProcessesRunsCreateRequest',
        response_type_name='GoogleCloudDatacatalogLineageV1Run',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes the run with the specified name.

      Args:
        request: (DatalineageProjectsLocationsProcessesRunsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleLongrunningOperation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/processes/{processesId}/runs/{runsId}',
        http_method='DELETE',
        method_id='datalineage.projects.locations.processes.runs.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['allowMissing'],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='DatalineageProjectsLocationsProcessesRunsDeleteRequest',
        response_type_name='GoogleLongrunningOperation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets the details of the specified run.

      Args:
        request: (DatalineageProjectsLocationsProcessesRunsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudDatacatalogLineageV1Run) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/processes/{processesId}/runs/{runsId}',
        http_method='GET',
        method_id='datalineage.projects.locations.processes.runs.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='DatalineageProjectsLocationsProcessesRunsGetRequest',
        response_type_name='GoogleCloudDatacatalogLineageV1Run',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists runs in the given project and location. List order is descending by `start_time`.

      Args:
        request: (DatalineageProjectsLocationsProcessesRunsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudDatacatalogLineageV1ListRunsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/processes/{processesId}/runs',
        http_method='GET',
        method_id='datalineage.projects.locations.processes.runs.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['pageSize', 'pageToken'],
        relative_path='v1/{+parent}/runs',
        request_field='',
        request_type_name='DatalineageProjectsLocationsProcessesRunsListRequest',
        response_type_name='GoogleCloudDatacatalogLineageV1ListRunsResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Updates a run.

      Args:
        request: (DatalineageProjectsLocationsProcessesRunsPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudDatacatalogLineageV1Run) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/processes/{processesId}/runs/{runsId}',
        http_method='PATCH',
        method_id='datalineage.projects.locations.processes.runs.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['allowMissing', 'updateMask'],
        relative_path='v1/{+name}',
        request_field='googleCloudDatacatalogLineageV1Run',
        request_type_name='DatalineageProjectsLocationsProcessesRunsPatchRequest',
        response_type_name='GoogleCloudDatacatalogLineageV1Run',
        supports_download=False,
    )

  class ProjectsLocationsProcessesService(base_api.BaseApiService):
    """Service class for the projects_locations_processes resource."""

    _NAME = 'projects_locations_processes'

    def __init__(self, client):
      super(DatalineageV1.ProjectsLocationsProcessesService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a new process.

      Args:
        request: (DatalineageProjectsLocationsProcessesCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudDatacatalogLineageV1Process) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/processes',
        http_method='POST',
        method_id='datalineage.projects.locations.processes.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['requestId'],
        relative_path='v1/{+parent}/processes',
        request_field='googleCloudDatacatalogLineageV1Process',
        request_type_name='DatalineageProjectsLocationsProcessesCreateRequest',
        response_type_name='GoogleCloudDatacatalogLineageV1Process',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes the process with the specified name.

      Args:
        request: (DatalineageProjectsLocationsProcessesDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleLongrunningOperation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/processes/{processesId}',
        http_method='DELETE',
        method_id='datalineage.projects.locations.processes.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['allowMissing'],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='DatalineageProjectsLocationsProcessesDeleteRequest',
        response_type_name='GoogleLongrunningOperation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets the details of the specified process.

      Args:
        request: (DatalineageProjectsLocationsProcessesGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudDatacatalogLineageV1Process) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/processes/{processesId}',
        http_method='GET',
        method_id='datalineage.projects.locations.processes.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1/{+name}',
        request_field='',
        request_type_name='DatalineageProjectsLocationsProcessesGetRequest',
        response_type_name='GoogleCloudDatacatalogLineageV1Process',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""List processes in the given project and location. List order is descending by insertion time.

      Args:
        request: (DatalineageProjectsLocationsProcessesListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudDatacatalogLineageV1ListProcessesResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/processes',
        http_method='GET',
        method_id='datalineage.projects.locations.processes.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['pageSize', 'pageToken'],
        relative_path='v1/{+parent}/processes',
        request_field='',
        request_type_name='DatalineageProjectsLocationsProcessesListRequest',
        response_type_name='GoogleCloudDatacatalogLineageV1ListProcessesResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Updates a process.

      Args:
        request: (DatalineageProjectsLocationsProcessesPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudDatacatalogLineageV1Process) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}/processes/{processesId}',
        http_method='PATCH',
        method_id='datalineage.projects.locations.processes.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['allowMissing', 'requestId', 'updateMask'],
        relative_path='v1/{+name}',
        request_field='googleCloudDatacatalogLineageV1Process',
        request_type_name='DatalineageProjectsLocationsProcessesPatchRequest',
        response_type_name='GoogleCloudDatacatalogLineageV1Process',
        supports_download=False,
    )

  class ProjectsLocationsService(base_api.BaseApiService):
    """Service class for the projects_locations resource."""

    _NAME = 'projects_locations'

    def __init__(self, client):
      super(DatalineageV1.ProjectsLocationsService, self).__init__(client)
      self._upload_configs = {
          }

    def BatchSearchLinkProcesses(self, request, global_params=None):
      r"""Retrieve information about LineageProcesses associated with specific links. LineageProcesses are transformation pipelines that result in data flowing from **source** to **target** assets. Links between assets represent this operation. If you have specific link names, you can use this method to verify which LineageProcesses contribute to creating those links. See the SearchLinks method for more information on how to retrieve link name. You can retrieve the LineageProcess information in every project where you have the `datalineage.events.get` permission. The project provided in the URL is used for Billing and Quota.

      Args:
        request: (DatalineageProjectsLocationsBatchSearchLinkProcessesRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudDatacatalogLineageV1BatchSearchLinkProcessesResponse) The response message.
      """
      config = self.GetMethodConfig('BatchSearchLinkProcesses')
      return self._RunMethod(
          config, request, global_params=global_params)

    BatchSearchLinkProcesses.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}:batchSearchLinkProcesses',
        http_method='POST',
        method_id='datalineage.projects.locations.batchSearchLinkProcesses',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=[],
        relative_path='v1/{+parent}:batchSearchLinkProcesses',
        request_field='googleCloudDatacatalogLineageV1BatchSearchLinkProcessesRequest',
        request_type_name='DatalineageProjectsLocationsBatchSearchLinkProcessesRequest',
        response_type_name='GoogleCloudDatacatalogLineageV1BatchSearchLinkProcessesResponse',
        supports_download=False,
    )

    def SearchLineageStreaming(self, request, global_params=None):
      r"""Retrieves a streaming response of lineage links connected to the requested assets by performing a breadth-first search in the given direction. Links represent the data flow between **source** (upstream) and **target** (downstream) assets in transformation pipelines. Links are stored in the same project as the Lineage Events that create them. This method retrieves links from all valid locations provided in the request. This method supports Column-Level Lineage (CLL) along with wildcard support to retrieve all CLL for an Entity FQN. Following permissions are required to retrieve links: * `datalineage.events.get` permission for the project where the link is stored for entity-level lineage. * `datalineage.events.getFields` permission for the project where the link is stored for column-level lineage. This method also returns processes that created the links if explicitly requested by setting [max_process_per_link](google.cloud.datacatalog.lineage.v1.SearchLineageStreamingRequest.limits.max_process_per_link) is non-zero and full process details are requested via `links.processes.process` in the [FieldMask](https://developers.google.com/workspace/docs/api/how-tos/field-masks#read_with_a_field_mask). Permission required to retrieve processes: * `datalineage.processes.get` permission for the project where the process is stored.

      Args:
        request: (DatalineageProjectsLocationsSearchLineageStreamingRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudDatacatalogLineageV1SearchLineageStreamingResponse) The response message.
      """
      config = self.GetMethodConfig('SearchLineageStreaming')
      return self._RunMethod(
          config, request, global_params=global_params)

    SearchLineageStreaming.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}:searchLineageStreaming',
        http_method='POST',
        method_id='datalineage.projects.locations.searchLineageStreaming',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=[],
        relative_path='v1/{+parent}:searchLineageStreaming',
        request_field='googleCloudDatacatalogLineageV1SearchLineageStreamingRequest',
        request_type_name='DatalineageProjectsLocationsSearchLineageStreamingRequest',
        response_type_name='GoogleCloudDatacatalogLineageV1SearchLineageStreamingResponse',
        supports_download=False,
    )

    def SearchLinks(self, request, global_params=None):
      r"""Retrieve a list of links connected to a specific asset. Links represent the data flow between **source** (upstream) and **target** (downstream) assets in transformation pipelines. Links are stored in the same project as the Lineage Events that create them. You can retrieve links in every project where you have the `datalineage.events.get` permission. The project provided in the URL is used for Billing and Quota.

      Args:
        request: (DatalineageProjectsLocationsSearchLinksRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GoogleCloudDatacatalogLineageV1SearchLinksResponse) The response message.
      """
      config = self.GetMethodConfig('SearchLinks')
      return self._RunMethod(
          config, request, global_params=global_params)

    SearchLinks.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1/projects/{projectsId}/locations/{locationsId}:searchLinks',
        http_method='POST',
        method_id='datalineage.projects.locations.searchLinks',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=[],
        relative_path='v1/{+parent}:searchLinks',
        request_field='googleCloudDatacatalogLineageV1SearchLinksRequest',
        request_type_name='DatalineageProjectsLocationsSearchLinksRequest',
        response_type_name='GoogleCloudDatacatalogLineageV1SearchLinksResponse',
        supports_download=False,
    )

  class ProjectsService(base_api.BaseApiService):
    """Service class for the projects resource."""

    _NAME = 'projects'

    def __init__(self, client):
      super(DatalineageV1.ProjectsService, self).__init__(client)
      self._upload_configs = {
          }



# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""API wrapper for `gcloud network-security mirroring-endpoint-groups` commands."""


import datetime

from apitools.base.py import list_pager
from googlecloudsdk.api_lib.util import apis
from googlecloudsdk.api_lib.util import waiter
from googlecloudsdk.calliope import base
from googlecloudsdk.core import resources


_API_VERSION_FOR_TRACK = {
    base.ReleaseTrack.ALPHA: 'v1alpha1',
    base.ReleaseTrack.BETA: 'v1beta1',
    base.ReleaseTrack.GA: 'v1',
}
_API_NAME = 'networksecurity'

_PACKET_BROKER_SUPPORTED = (
    base.ReleaseTrack.ALPHA,
    base.ReleaseTrack.BETA,
)


def GetMessagesModule(release_track=base.ReleaseTrack.ALPHA):
  api_version = GetApiVersion(release_track)
  return apis.GetMessagesModule(_API_NAME, api_version)


def GetClientInstance(release_track=base.ReleaseTrack.ALPHA):
  api_version = GetApiVersion(release_track)
  return apis.GetClientInstance(_API_NAME, api_version)


def GetEffectiveApiEndpoint(release_track=base.ReleaseTrack.ALPHA):
  api_version = GetApiVersion(release_track)
  return apis.GetEffectiveApiEndpoint(_API_NAME, api_version)


def GetApiVersion(release_track=base.ReleaseTrack.ALPHA):
  return _API_VERSION_FOR_TRACK.get(release_track)


class Client:
  """API client for Mirroring Endpoint Groups commands.

  Attributes:
    messages: API messages class, The Mirroring Endpoint Groups API messages.
  """

  def __init__(self, release_track):
    self._release_track = release_track
    self._client = GetClientInstance(release_track)
    self._endpoint_group_client = (
        self._client.projects_locations_mirroringEndpointGroups
    )
    self._operations_client = self._client.projects_locations_operations
    self.messages = GetMessagesModule(release_track)
    self._resource_parser = resources.Registry()
    self._resource_parser.RegisterApiByName(
        _API_NAME, GetApiVersion(release_track)
    )

  def CreateEndpointGroup(
      self,
      endpoint_group_id,
      parent,
      deployment_groups,
      description,
      endpoint_group_type='DIRECT',
      labels=None,
  ):
    """Calls the CreateEndpointGroup API.

    Args:
      endpoint_group_id: The ID of the Endpoint Group to create.
      parent: The parent of the Endpoint Group to create.
      deployment_groups: The Mirroring Deployment Group(s) to associate with the
        Endpoint Group. Can be a single string or a list of strings.
      description: Description of the Endpoint Group.
      endpoint_group_type: Type of the Endpoint Group (DIRECT or BROKER).
      labels: Labels to apply to the Endpoint Group.

    Returns:
      Operation ref to track the long-running process.
    """

    endpoint_group = self.messages.MirroringEndpointGroup(
        labels=labels,
        description=description,
    )

    if self._release_track in _PACKET_BROKER_SUPPORTED:
      if not endpoint_group_type:
        endpoint_group_type = 'DIRECT'

      endpoint_group.type = (
          self.messages.MirroringEndpointGroup.TypeValueValuesEnum(
              endpoint_group_type
          )
      )
      if endpoint_group_type == 'BROKER':
        if isinstance(deployment_groups, list):
          endpoint_group.mirroringDeploymentGroups = deployment_groups
        else:
          endpoint_group.mirroringDeploymentGroups = [deployment_groups]
      elif endpoint_group_type == 'DIRECT':
        endpoint_group.mirroringDeploymentGroup = deployment_groups
      else:
        raise ValueError(
            f'Unsupported endpoint group type: {endpoint_group_type}'
        )
    else:
      endpoint_group.mirroringDeploymentGroup = deployment_groups

    create_request = self.messages.NetworksecurityProjectsLocationsMirroringEndpointGroupsCreateRequest(
        mirroringEndpointGroup=endpoint_group,
        mirroringEndpointGroupId=endpoint_group_id,
        parent=parent,
    )
    return self._endpoint_group_client.Create(create_request)

  def DeleteEndpointGroup(self, name):
    """Calls the DeleteEndpointGroup API."""
    delete_request = self.messages.NetworksecurityProjectsLocationsMirroringEndpointGroupsDeleteRequest(
        name=name
    )
    return self._endpoint_group_client.Delete(delete_request)

  def UpdateEndpointGroup(self, name, description, update_fields):
    """Calls the UpdateEndpointGroup API.

    Args:
      name: The name of the Endpoint Group to update.
      description: Description of the Endpoint Group.
      update_fields: A dictionary of the fields to update mapped to their new
        values.

    Returns:
      Operation ref to track the long-running process.
    """
    endpoint_group = self.messages.MirroringEndpointGroup(
        labels=update_fields.get('labels', None),
        description=description,
    )

    update_request = self.messages.NetworksecurityProjectsLocationsMirroringEndpointGroupsPatchRequest(
        name=name,
        mirroringEndpointGroup=endpoint_group,
        updateMask=','.join(update_fields.keys()),
    )
    return self._endpoint_group_client.Patch(update_request)

  def DescribeEndpointGroup(self, name):
    """Calls the GetEndpointGroup API."""
    get_request = self.messages.NetworksecurityProjectsLocationsMirroringEndpointGroupsGetRequest(
        name=name
    )
    return self._endpoint_group_client.Get(get_request)

  def ListEndpointGroups(
      self, parent, limit=None, page_size=None, list_filter=None
  ):
    """Calls the ListEndpointGroups API."""
    list_request = self.messages.NetworksecurityProjectsLocationsMirroringEndpointGroupsListRequest(
        parent=parent, filter=list_filter
    )
    return list_pager.YieldFromList(
        self._endpoint_group_client,
        list_request,
        batch_size=page_size,
        limit=limit,
        field='mirroringEndpointGroups',
        batch_size_attribute='pageSize',
    )

  def GetOperationRef(self, operation):
    """Converts an Operation to a Resource that can be used with `waiter.WaitFor`."""
    return self._resource_parser.ParseRelativeName(
        operation.name, 'networksecurity.projects.locations.operations'
    )

  def WaitForOperation(
      self,
      operation_ref,
      message,
      has_result=True,
      max_wait=datetime.timedelta(seconds=600),
  ):
    """Waits for an operation to complete.

    Polls the Network Security Operation service until the operation completes,
    fails, or max_wait_seconds elapses.

    Args:
      operation_ref: A Resource created by GetOperationRef describing the
        Operation.
      message: The message to display to the user while they wait.
      has_result: If True, the function will return the target of the operation
        (the Mirroring Endpoint Group) when it completes. If False, nothing will
        be returned (useful for Delete operations)
      max_wait: The time to wait for the operation to succeed before timing out.

    Returns:
      if has_result = True, an Endpoint Group entity.
      Otherwise, None.
    """
    if has_result:
      poller = waiter.CloudOperationPoller(
          self._endpoint_group_client, self._operations_client
      )
    else:
      poller = waiter.CloudOperationPollerNoResources(self._operations_client)

    return waiter.WaitFor(
        poller,
        operation_ref,
        message,
        max_wait_ms=int(max_wait.total_seconds()) * 1000,
    )
